/*
Controller for GSM autonome module
with processor AT89C4051
clock frequency - 11.059 MHz one machine cycle = 1.085 uS
serial port		- 9600 - 8 - N - 1

version 22.11.2000  18:30
------- feature of disconnect detect --------
------- table structure ---------------------
------- tested - OK 10.11.2000 - final working release
------- possible changes only in LED control (blinking, etc,)
------- 22.11.2000 - added features for MUTE when power OFF,smart RING detect, avoid ATA problem
------- 30.03.2001 - added commands for better adjustment
------- 02.04.2001 - REMOVED PROBLEM WITH ATA
------- 02.04.2001 - added delay and command for RING problem
------- 02.04.2001 - FINAL VERSION !!!!!!
------- 04.04.2001 - REMOVED BUG WITH long pressed key1&2 - now this will not switch ON/OFF the module
------- 14.06.2001 - new I/O pin definitions
------- 24.06.2001 - implementation of new idea for flags
------- 24.06.2001 - removed bug with limitation of table
------- 25.06.2001 - some changes of algorythm - if conversation - power off don't work
------- 25.06.2001 - implementation of new pseudorandom command detection from DALLAS
------- 26.06.2001 - removed problem with command overflows - command stack implemented
------- 27.06.2001 - removed bug with hanging 
------- 09.10.2001 - codes over 0x80 - skipped, make all letters caps 
------- 09.10.2001 - removed bug with no response?
------- 09.10.2001 - removed wrong control of MUTE, POWER ON, etc = P1, P3 image control was failed.
------- 09.10.2001 - Seems to be BUGFREE VERSION!!!! at last... but still errors
------- 12.10.2001 - changed HOLD algorythm in command() to recover errors similar to ATA




*/


#include <reg51.h>		// main library - description of pins

typedef unsigned char byte;
typedef unsigned int word;


void com_init(void);					  		// initialization of communication port
void TxD_byte(byte danna);					// send one byte data via RS232 port
//void debug(byte data1,word data2);   	// debugger routine

unsigned static bdata char PORT1_image, PORT3_image;		// image of ports to control outputs

/*************************************************************************/
// definition of I/O bits
/*************************************************************************/
sbit RxD		=	P3^0;					// RxD RS232 line
sbit TxD		=	PORT3_image^1;		// TxD RS232 line out

sbit DTR		=	PORT1_image^7;		// DTR RS232 output line
sbit DSR		=	P1^6;					// DSR RS232 input line 		initial state 1
sbit IGNITION= PORT1_image^5;		// Ignition 
sbit RTS		=	PORT1_image^4;		// RTS output RS232 line
sbit DONT_USE=	PORT1_image^3;		// RI  RS232 line
sbit CTS		=	P1^2;					// CTS RS232 input line
sbit GSM_ON	=	P1^1;					// GSM IS switched ON
sbit POWER_ON= PORT1_image^0;		// POWER_ON 
sbit MUTE 	=	PORT3_image^7;		// MUTE line
sbit RING_LED= PORT3_image^5;		// LED - RING
sbit CLOSE_SW= P3^4;		// SWITCH - CLOSE
sbit OPEN_SW =	P3^3;		// SWITCH - OPEN
sbit POWER_LED=PORT3_image^2;		// LED of power




/*************************************************************************/
// definition of message etalons and their corresponding flag_bits
/*************************************************************************/

#define out0_1 0b01001110						// define value in whish all outputs go to 0 for PORT1
#define out0_3 0b11011011						// define value in whish all outputs go to 0 for PORT3 TxD and RxD - high
#define one_minute 600							// delay for checking system every minute
#define mask_POWER_OFF 0b00000100			// both keys pressed for longer then 2s
#define ring_time	50								// 5 s maximum time between rings
#define cmnd_time 10								// 1 s to wait response
#define middle_cmnd_time 25
#define long_cmnd_time 50						// 3s for long operations
#define period_of_check_ringing 6			// 0.5s to check the ring status

#define time_100ms	3							// time to wait to free port	- 0.2 - 0.3s


code word LED0 = 0;									// mask for LED control 1 - DARK
code word LED1 = 0xFFFF;						 	// LIGHT
code word LED2 = 0b1111000011110000;			// DARK 0.4S LIGHT 0.4S
code word LED3 = 0b1010000010100000;			// 2 times x100ms lights pause 600ms
code word LED4 = 0b1100110011001100;			// 200ms light 200ms dark
code word LED5 = 0b0011001100110011;			// 200ms dark  200ms light

							// NO MESSAGE FOR sending, just to imitate sending 
code byte EMPTY[] = {0x00};

							// ATH MESSAGE FOR sending
						// ATH MESSAGE FOR sending
code byte ATH[] = "ATH\x0D\x0A";

							// ATH MESSAGE FOR sending 
code byte ATA[] = "ATA\x0D\x0A";

							// ATcheck FOR RINGING - TO ENSURE THAT RING IS STILL ACTIVE 
code byte ATCHECK_STATUS[] = "AT+CPAS\x0D\x0A";

							// ATHOLD MESSAGE FOR sending 
code byte ATHOLD[] = "AT+CHLD=2\x0D\x0A";

							// ATSMS MESSAGE FOR sending 
code byte ATSMSO[] = "AT^SMSO\x0D\x0A";

							// ATCBC MESSAGE FOR sending 
code byte ATCBC[] =  "AT+CBC\x0D\x0A";

                     // set in custom audio mode
code byte ATVIP[] =  "AT+VIP=3\x0D\x0A";

code byte ATSETUP[] = "AT^SNFO=2,5792,8192,11585,16384,23170,32768,46340,65535,3,0\x0D\x0A";                    // END OF THE STRING

code byte ASK_FOR_PIN[] 		= "AT+CPIN?\x0A\x0D";     // ask if there is pin code waiting
code byte ASK_PIN_ATTEMPTS[] 	= "AT^SPIC\x0A\x0D";      // ask if there is pin code waiting
code byte ENTER_PIN_CODE[]		= "AT+CPIN=0000\x0A\x0D"; // enter PIN code (factory set to 000)



//--------------------- ETALONS FOR COMPARING DATA -------------------------
//=========== ARRAY FOR MAKING CRC ON COMING DATA ===========================
code byte TABLE_LOW[]	= { 
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x00,0xC1,0x81,0x40,0x01,0xC0,0x80,0x41,
0x01,0xC0,0x80,0x41,0x00,0xC1,0x81,0x40 };


code byte TABLE_HIGH[]	= {
0x00,0xC0,0xC1,0x01,0xC3,0x03,0x02,0xC2,
0xC6,0x06,0x07,0xC7,0x05,0xC5,0xC4,0x04,
0xCC,0x0C,0x0D,0xCD,0x0F,0xCF,0xCE,0x0E,
0x0A,0xCA,0xCB,0x0B,0xC9,0x09,0x08,0xC8,
0xD8,0x18,0x19,0xD9,0x1B,0xDB,0xDA,0x1A,
0x1E,0xDE,0xDF,0x1F,0xDD,0x1D,0x1C,0xDC,
0x14,0xD4,0xD5,0x15,0xD7,0x17,0x16,0xD6,
0xD2,0x12,0x13,0xD3,0x11,0xD1,0xD0,0x10,
0xF0,0x30,0x31,0xF1,0x33,0xF3,0xF2,0x32,
0x36,0xF6,0xF7,0x37,0xF5,0x35,0x34,0xF4,
0x3C,0xFC,0xFD,0x3D,0xFF,0x3F,0x3E,0xFE,
0xFA,0x3A,0x3B,0xFB,0x39,0xF9,0xF8,0x38,
0x28,0xE8,0xE9,0x29,0xEB,0x2B,0x2A,0xEA,
0xEE,0x2E,0x2F,0xEF,0x2D,0xED,0xEC,0x2C,
0xE4,0x24,0x25,0xE5,0x27,0xE7,0xE6,0x26,
0x22,0xE2,0xE3,0x23,0xE1,0x21,0x20,0xE0,
0xA0,0x60,0x61,0xA1,0x63,0xA3,0xA2,0x62,
0x66,0xA6,0xA7,0x67,0xA5,0x65,0x64,0xA4,
0x6C,0xAC,0xAD,0x6D,0xAF,0x6F,0x6E,0xAE,
0xAA,0x6A,0x6B,0xAB,0x69,0xA9,0xA8,0x68,
0x78,0xB8,0xB9,0x79,0xBB,0x7B,0x7A,0xBA,
0xBE,0x7E,0x7F,0xBF,0x7D,0xBD,0xBC,0x7C,
0xB4,0x74,0x75,0xB5,0x77,0xB7,0xB6,0x76,
0x72,0xB2,0xB3,0x73,0xB1,0x71,0x70,0xB0,
0x50,0x90,0x91,0x51,0x93,0x53,0x52,0x92,
0x96,0x56,0x57,0x97,0x55,0x95,0x94,0x54,
0x9C,0x5C,0x5D,0x9D,0x5F,0x9F,0x9E,0x5E,
0x5A,0x9A,0x9B,0x5B,0x99,0x59,0x58,0x98,
0x88,0x48,0x49,0x89,0x4B,0x8B,0x8A,0x4A,
0x4E,0x8E,0x8F,0x4F,0x8D,0x4D,0x4C,0x8C,
0x44,0x84,0x85,0x45,0x87,0x47,0x46,0x86,  
0x82,0x42,0x43,0x83,0x41,0x81,0x80,0x40 };


//=============== DEFINITTIONS OF THE ETALONS

#define _CR_LF				0x54D5		// empty CR_LF instruction

#define _OK 				0x2363		// OK _CR _LF message CRC

#define _ERROR				0x3878		// ERROR _CR _LF message CRC				

#define _NO_CARRIER		0x6CAD		// NO CARRIER _CR _LF message CRC

#define _RNG_STAT0		0x37B6		// +CPAS: 0 _CR _LF message CRC

#define _RNG_STAT1		0x0B4B		// +CPAS: 1 _CR _LF message CRC

#define _RNG_STAT2		0x5B1B		// +CPAS: 2 _CR _LF message CRC

#define _RNG_STAT3		0x67E6		// +CPAS: 3 _CR _LF message CRC

#define _RNG_STAT4		0xCB4B		// +CPAS: 4 _CR _LF message CRC

#define _RING_ASYNC		0x4F0F		// +CRING: ASYNC _CR _LF message CRC

#define _RING_SYNC		0x8505		// +CRING: SYNC _CR _LF message CRC

#define _RING_RASYNC		0x5C9C		// +CRING: REL ASYNC _CR _LF message CRC

#define _RING_RSYNC		0xBFFF		// +CRING: REL SYNC _CR _LF message CRC

#define _RING_FAX			0x23A2		// +CRING: FAX _CR _LF message CRC

#define _RING_VOICE		0x9EDF		// +CRING: VOICE _CR _LF message CRC
 							   
#define _CREG0				0x787A		// +CREG: 0 _CR _LF message CRC

#define _CREG1				0x4787 		// +CREG: 1 _CR _LF message CRC

#define _CREG2				0x17D7		// +CREG: 2 _CR _LF message CRC

#define _CREG3				0x2B2A		// +CREG: 3 _CR _LF message CRC

#define _CREG4				0x8787		// +CREG: 4 _CR _LF message CRC

#define _CREG5				0xBB7A		// +CREG: 5 _CR _LF message CRC


#define _NO_CARD			0xBAFA		// ^SCKS: 0 _CR _LF message CRC

#define _CARD				0x8607		// ^SCKS: 1 _CR _LF message CRC

#define _NO_PIN			0xCD8D		// +CPIN: READY _CR _LF message CRC

#define _PIN_NEED			0xF8B9		// +CPIN: SIM PIN _CR _LF message CRC

#define _NO_ATTEMPT		0xC6C6		// ^SPIC:1 _CR _LF message CRC

#define _CAN_ATTEMPT2	0x9696		// ^SPIC:2 _CR _LF message CRC

#define _CAN_ATTEMPT3	0xAA6B		// ^SPIC:3 _CR _LF message CRC



// here could be made some speed optimization - as closer to the beginning of the table are patterns
// then faster is the process of calculations



#define length_of_FIFO 4							// 5 level command STACK
static word *pointer_HEAD,*pointer_TAIL;		// stack FIFO for commands
static word COMMAND_BUFFER[length_of_FIFO];	// array for stack


static code byte *point_value;   // keep sbegin address of the pattern to send
static byte time_outer;						// to calculate time out for sending data
static byte module_number;					// cell that switches to different modules
static word bdata flags;					// array of flags, used in the main program, bit addressable

/* ------------ description of NEW FLAGS - they are used in commands, and they hold
 					MORE IMPORTANT INFORMATION
 					
after receieving data from RxD finishing with CR LF, command processor analyzes the incomming data
corresponding bits are set in flags register, they are only set, software have to reset them after

+-----------------------------------------------+-----------------------------------------------+
|     |     |CAN  |NO   | PIN | NO  |CARD |NO   |REGIS|NOT  |CRING|CRING|RING |NO CA|ERROR| O K | 
|     |     |ATTEM|ATTEM| NEED| PIN |INSER|CARD |TERED|REGIS|OTHER|VOICE|END  |RRIER|     |     | 
+-----------------------------------------------+-----------------------------------------------+
  15    14    13    12    11    10    09    08    07    06    05    04    03    02    01    00            

*/
#define clear_flags	0b0000000100000000;	// clears all except card


// because the data is high byte, low byte there is some changes of addressing
#define mask_REGISTERED		0x80			// GSM registered
#define mask_ring_voice		0x10			// voice ring
#define mask_ring_no_voice	0x20			// no voice ring
#define mask_NO_CONNECTION 0x44			// CREG0,2,3,4,5,NO_CARRIER
#define mask_NO_CARRIER		0x04			// no carrier detected
#define mask_NO_SIM			0x100			// no card inserted
#define mask_EXISTS_SIM		0x200       // sim card inserted
#define mask_NO_PIN_CODE	0x400			// NO PIN CODE NEEDED
#define mask_PIN_CODE_NEED 0x800       // needs PIN code
#define mask_NO_ATTEMPTS	0x1000      // no available attempts
#define mask_CAN_ATTEMPT	0x2000		// can attempt PIN codes


sbit flag_OK = 				flags ^ 8;	// if =1 OK message appeared
sbit flag_ERROR = 			flags ^ 9;	// if =1 ERROR message appeared
sbit flag_NO_CARRIER = 		flags ^10;	// if =1 NO CARRIER message appeared
sbit flag_RING_CHECK = 		flags ^11;	// if =1 the ring is not active anymore
sbit flag_RING_VOICE = 		flags ^12;	// if =1 voice ring 
sbit flag_RING_OTHER = 		flags ^13; 	// if =1 other rings came
sbit flag_NOT_REGISTERED= 	flags ^14;	// if =1 registration in progress, but still not registered
sbit flag_REGISTERED = 		flags ^15;	// if =1 registered in the net
sbit flag_NO_CARD = 			flags ^ 0;	// if =1 no SIM card detection
sbit flag_CARD_INSERTED = 	flags ^ 1;	// if =1 SIM card detected
sbit flag_NO_PIN = 			flags ^ 2 ;	// if =1 no need to enter PIN code
sbit flag_PIN_NEEDED = 		flags ^ 3 ;	// if =1 PIN code needed, should be only 0000
sbit flag_NO_ATTEMPTS =		flags ^ 4 ;	// if =1 no available attempts (1) - don't try anymore
sbit flag_CAN_ATTEMPT = 	flags ^ 5 ;	// if =1 2 or 3 available attempts



#define number_of_commands 27
#define ring_voice_number 11		// to detect if voice ring occured

code word PATTERNS[number_of_commands*2 +2] = {
_ERROR,			0x0002,
_NO_CARRIER,	0x0004,
_RNG_STAT0,		0x000C,
_RNG_STAT1, 	0x000C,
_RNG_STAT2, 	0x000C,
_RNG_STAT4, 	0x0008,
_RING_ASYNC,	0x0020,
_RING_SYNC,		0x0020,
_RING_RASYNC,	0x0020,
_RING_RSYNC,	0x0020,
_RING_FAX,   	0x0020,
_RING_VOICE, 	0x0010,
_CREG0,      	0x0040,
_CREG2,      	0x0040,
_CREG3,      	0x0040,
_CREG4,			0x0040,
_CREG5,    		0x0040,
_CREG1,    		0x0080,
_NO_CARD,   	0x0100,
_CARD,       	mask_EXISTS_SIM,
_NO_PIN,      	mask_NO_PIN_CODE,
_PIN_NEED,    	mask_PIN_CODE_NEED,
_NO_ATTEMPT,  	mask_NO_ATTEMPTS,
_CAN_ATTEMPT2,	mask_CAN_ATTEMPT,
_CAN_ATTEMPT3, mask_CAN_ATTEMPT,
_OK,  			0x0001,
_RNG_STAT3, 	0x0000,   // no carrier if ringing
0x0000,0x0000				};




//-----------DEFINITION  OF THE BRANCH TABLE ------------------------------

#define	POINT_3		0			
#define	POINT_C		1			
#define  POINT_FN		2  
#define  POINT_F		3  
#define	POINT_B		4			
#define	POINT_R		5			
#define  POINT_N		6   // pin support
#define  POINT_N1		7   // pin support
#define  POINT_N2		8   // pin support
#define  POINT_N3		9   // pin support
#define  POINT_NA 	10	 // used in PIN support
#define  POINT_NA1 	11	 // used in PIN support
#define  POINT_NA2 	12	 // used in PIN support
#define	MAIN_LOOP 	13
#define	MAIN_LOOP1 	14
#define	POINT_P		15		
#define	POINT_X		16 	
#define	POINT_X1		17		
#define	POINT_2		18		
#define	POINT_U		19		
#define	POINT_U1		20		
#define	POINT_J		21		
#define	POINT_K		22		
#define	POINT_K1		23		
#define  POINT_L  	24
#define  POWER_OFF	25






/*************************************************************************/
// definition of message etalons and their corresponding flag_bits
/*************************************************************************/

static bit flag_noresponse;				// no response
static bit HOLD_exception;					// if =1 how should initialize HOLD command - could fail,
													// so +CPAS3 will set nocarrier flag

static byte flag_command_received;		// if !=0 - begins within 10ms command processing
static word COMMANDO;					// keeps the CRC
static byte	CRC_high,CRC_low;		// where to build CRC

static bit	flag_debug = 1;		// @@@@@@@@@@@@@@@

static bit  flag_pressed = 0;				// both keys pressed disable multiple signalling of flag
static bit	flag_both_pressed_2s = 0;	// both keys were pressed for longer then 2s					
static byte timer_pressed_2s = 0;		// check how long the both keys were pressed timer
static word RING_pattern, POWER_pattern; // in pattern is kept the current value of LED sceme, mask scans it
static byte	LED_prescaller;				// makes 1:10 prescale to allow 100ms INDIC timer
static word LED_mask;						// in pattern is kept the current value of LED sceme, mask scans it

static word timer_check_every_minute;	// generates bit set of check_every_minute every 60s
static bit	check_every_minute;			// if = 1 have to check is the device is OK or not

static byte timeout_counter;				// counter for timeouts x 100ms
static byte check_ring;						// counter for common purpose -  decrements every 0.1s

static byte danni_old;						// keeps old value of danni (in INT routine RxD)

static byte timer_waiter;						// timer for waiter delay features
 
static bit close,old_close,open,old_open;	// bit where to keep read value from keys
static bit CLOSE_SW_pressed;					// flag that key was pressed
static bit OPEN_SW_pressed;					// flag that key was pressed

static bit RING_active;							// flag that shows active ring. If rings were not comming within ring_time - flag is reset
														// after the ring it stays =1 time ring_time	

static byte ring_timer;							// waits ring_time before clears RING_active


static bit flag_CR;								// keeps track of previous reading a value CR 
static bit flag_TxD_finish;					// =0 after finishing the TxD transfere

static bit flag_ONLINE;							// keeps track of connect/disconnect events 

static bit flag_POINT_B;						// used for GOTO imitations
static bit module_ON;							// if =1 the modul i sswithed on

//================== INITIAL DECLARATION OF FUNCTIONS ========================
void wait(byte time_wait);  // wait <time_wait> x 100ms time



/**************************************************************/
void com_init(void)  // initialization of communication port
/**************************************************************/
{
	 SCON=0b01010010;  /* 8 BIT serial MOD-1, ENABLE RECEPTION */
	 TMOD=0b00100001;  /* TIMER 1 will be in mode  8 BIT AUTORELOAD */
	 PCON=0x0;  /* DOUBLE RATE BIT - timer 1 will not generate double frequency  bit 7 of PCON=0) */
	 TH1=0xfd;   /* 256-((2*OSC)/(384*19200)) = 256-3 ---- will produce 19200*/
	 TR1=1;      /* START TIMER 1 */

	/* ------- setting TIMER0 interrupt ---------- */
	
	 TH0 = 220;					// initial value to produce INT every 10 mS
	 TL0 = 0;					// low byte of the value
	 
	 TR0 = 1;					// start TIMER0 operation




	/* ------- setting Interrupt ---------- */
	 PS = 1;				// defines serial port high priority of interrupts	
	 REN = 1;			// enable reception from serIal port
	 ES = 1;				// enable serial port interrupts
	 ET0 = 1;			// enables Interrupt from TIMER0 overflow
	
	 IP = 0;
//	 PS = 1;	         // set highest priority level to RxD TxD interrupts 
							// din't put it to 1 - system hangs

	 EA = 1;				// enable all interrupts to be individually acknowleged
	 
	/* ----------------- */

}




/*************************************************************************/
void TxD_byte(byte danna)		// send one byte data via RS232 port
/*************************************************************************/
{
	flag_TxD_finish = 0;					// IF TI->1 means the bit is transmitted
	SBUF=danna;								// send byte
			
  	while(!flag_TxD_finish);			// wait until buffer becomes empty
	
	TI=0;										// clears interrupt flag


}






/*************************************************************************/
//void debug(byte data1,word data2)   // debugger routine
/*************************************************************************/
/*{
// sends first data1 , then data2 in hex format, spase between them and CR LF after both

	static char dan, info1;
	static int info2;

	info1 = data1;	
	info2 = data2;

	dan = ((info1 >> 4)&0x0F)+0x30;
	if (dan > 0x39)
		dan +=7;			// convert to ASCI

	TxD_byte(dan);
	
	
	dan = (info1 & 0x0F) + 0x30;
	if (dan > 0x39)
		dan +=7;			// convert to ASCI

	TxD_byte(dan);

	TxD_byte(0x20);


	dan = ((info2 >> 12) & 0x0F) + 0x30;
	if (dan > 0x39)
		dan +=7;			// convert to ASCI

	TxD_byte(dan);
	
	dan = ((info2 >> 8) & 0x0F) + 0x30;
	if (dan > 0x39)
		dan +=7;			// convert to ASCI

	TxD_byte(dan);


	dan = ((info2 >> 4) & 0x0F) + 0x30;
	if (dan > 0x39)
		dan +=7;			// convert to ASCI

	TxD_byte(dan);



	dan = (info2 & 0x0F) + 0x30;
	if (dan > 0x39)
		dan +=7;			// convert to ASCI

	TxD_byte(dan);

	TxD_byte(0x0D);
	TxD_byte(0x0A);


}               
*/

#define TAMNO 1
#define SWETI 0
/*************************************************************************/
//void debug_blink(unsigned char datas)   // debugger routine
/*************************************************************************/
/*{	

	// makes ligh blinks for the word

	static byte i, datar;
	datar = datas;
	
	
	DTR = TAMNO;					// dark led
	P1 = PORT1_image;				// set output

   timeout_counter = 15;		// dark LEDs for 1s
	while(timeout_counter);		// wait 1s darkeness

	
	for (i=0;i<8;i++)
	 {
	 	DTR = SWETI;
		P1 = PORT1_image;				// set output
		
	   timeout_counter = 1;			// LEDs for 0.1s
		while(timeout_counter);		// wait 	 	
	 
	 	DTR = TAMNO;
		P1 = PORT1_image;				// set output
		
	   timeout_counter = 2;			// LEDs for 0.1s
		while(timeout_counter);		// wait 	 	

		if (datar & 0x80)
		 {
		 	DTR = SWETI;
			P1 = PORT1_image;				// set output
		
		   timeout_counter = 1;			// LEDs for 0.1s
			while(timeout_counter);		// wait 	 	
	 
		 	DTR = TAMNO;
			P1 = PORT1_image;				// set output
		
		   timeout_counter = 2;			// LEDs for 0.1s
			while(timeout_counter);		// wait 	 	
		 }

		   timeout_counter = 5;			// LEDs for 0.5s
			while(timeout_counter);		// wait 	 	
				 
			datar <<= 1;						// shift mask	 
	 
	 }		

		   timeout_counter = 10;			// LEDs for 0.5s
			while(timeout_counter);		// wait 	 	

		 	DTR = 0;                   // restore initial settings
			P1 = PORT1_image;				// set output
 			

}

*/



/*************************************************************************/
byte command( byte time,code byte *pointa,byte online_cmnd)	// sends one command "how many times" times waiting response OK
/*************************************************************************/
// sends command pointed from "pointa' and wait within desired time in 0.1s for response OK (sometimes ERROR)
// if not OK tries "how_manY_times" to do the same. 
// if no response OK received, but after sending ATSMSO was OK - returns 1	-> command failed, but device still responding
// if everything successfull  -- returns 0
// mask keep track of other event to appear while waiting (like NO_CARRIER)


#define TYPE_HOLD	1
#define TYPE_ATA  2
#define TYPE_ONCE	3
{
	static byte i,j,wait_time;   	// keeps data for location of pattern
	static byte command_type;
	
   command_type = online_cmnd;			// if =1  if disconnection detected - skip command
   											// if =2 - like online command and error appear - treat it like OK - return without error
   											// if =3 - only once send the command
											
/*
		SOME EXCEPTIONS OF THE STANDARD COMMAND-RESPONSE ALGORYTHM
		0 - NO EXCEPTION 	
		1 - IF DISCONNECTION DETECTED - skip the command - ATH, ATHOLD
		2 - if online command and ERROR response - continue, despite that response is ERROR, ATA command
		3 - send command only once, used in PIN code commands
*/




   
	wait_time = time;				// keeps record for how many times to try
	
	
	if (!wait_time)   			// at least wait one tick 100ms
		wait_time++;	

	j = 5;      					// bring how many attempts to make

	if (command_type == TYPE_ONCE)		// send it only once
	  j = 1;
	  
	  
	for (i=0;i<j;i++)	// loop for how many times to check
		{

//--------!!!! this was the place where error appeared - commands were sent only once
//-------- the pointer that point sto the first byte of the message should be reloaded every attemp
			
			point_value = pointa;				// transferes data to be recorded

 			flag_OK = 0;									// flag for no error cleared
			flag_ERROR = 0;								// clears flag for ERROR
		
			if ((!flag_ONLINE || (flag_NO_CARRIER && command_type == TYPE_HOLD)) &&
					 (command_type == TYPE_HOLD || command_type == TYPE_ATA)) // check if disconnection found
				{
					flag_OK = 1;							// fake flag_OK to skip wrong operations (error could be accepted)
  		         break;                           // this operation is not valid when NOT ONLINE
            }
            

//------------ transfere DATA TO RS232
		
			while (*point_value)						// send bytes until 00 occure								
				{
   	
					time_outer = time_100ms;
					while (((CTS) || (DSR)) && (time_outer));		// wait to release the line, sometimes it could take up to 1ms
														// to begin transmission it should be = 0	
														// DSR should be set to 0 
														// if something wrong - will not wait longer then 65000 cyccles
														

			 		flag_TxD_finish = 0;					// IF TI->1 means the bit is transmitted
		 		
					SBUF=*point_value;						// send byte
			
      		 	while(!flag_TxD_finish);			// wait until buffer becomes empty
	
					point_value++;								// pointer to the next byte in message
				}

			TI=0;												// clears interrupt flag
	
//------------ END OF transfere DATA TO RS232
        

	// ------- timeout counter is decrement every 100ms
	
	      timeout_counter = wait_time;									// wait x500ms time
            
         while(timeout_counter && !flag_OK && !flag_ERROR);		// wait the end of timeout or OK receive, or ERROR
															
																					// if ERROR - retransmit the command immediately
			if (flag_OK || (flag_ERROR &&(command_type == TYPE_ATA || command_type==TYPE_ONCE)))		// check for additional event, or OK
				break;										// stop the loop
				
				
				
//------------- removing the bug from 12.10.2001 - when HOLD hangs
			
			if (flag_ERROR && command_type == TYPE_HOLD)				// if error appeared while HOLD command
			 {

				HOLD_exception = 1;								// will make +CPAS3 to make no carrier flag =1 
			 
           	point_value = ATCHECK_STATUS;					// sends ATCPAS commanf to determine the status, maybe no carrier
				while (*point_value)						// send bytes until 00 occure								
					{
   	
						time_outer = time_100ms;
						while (((CTS) || (DSR)) && (time_outer));		// wait to release the line, sometimes it could take up to 1ms
														// to begin transmission it should be = 0	
														// DSR should be set to 0 
														// if something wrong - will not wait longer then 65000 cyccles
														

			 			flag_TxD_finish = 0;					// IF TI->1 means the bit is transmitted
		 		   	
						SBUF=*point_value;						// send byte
			
   	   		 	while(!flag_TxD_finish);			// wait until buffer becomes empty
	
						point_value++;								// pointer to the next byte in message
					}


//-------------------- the command is already sent, now wait for response +CPAS0..4
//							  if CPAS4 - should repete the command - maybe some failure, otherwise - STOP
//							  and change the status of LEDs


				flag_ERROR = 0;								// clear error	

		      timeout_counter = wait_time;		// wait 500ms time
            

         while(timeout_counter && !flag_OK && !flag_ERROR);		// wait the end of timeout or OK receive, or ERROR

			HOLD_exception = 0;								// end of make +CPAS3 to make no carrier flag =1 
			if (flag_NO_CARRIER)
			 {								// fake that command
			 	flag_OK = 1;
			 	break;
			 }

        }
				
  																// if error - don't wait the whole time - reattempt sending command
		}  // loop to send command several times

   i=0;
   
   //===================== case when no OK received within 5 attempts =============
   
	if (!(flag_OK || (flag_ERROR && (command_type == TYPE_ATA || command_type==TYPE_ONCE))))			// 5 times was no success to send the command with OK response, or ERROR response
		{
		  IGNITION = 0;									// READY TO switch off the module
		  MUTE =1;											// mute = 1 to avaoid noices			  
		  P1 = PORT1_image;
		  P3 = PORT3_image;

		  wait(5);											// wait 0.5s

		  POWER_ON = 0;	
		  P1 = PORT1_image;
		  
		  for (i=0;i<5;i++)
		 	{


//------------ transfere DATA TO RS232

				point_value = &ATSMSO;									// send ATSMSO command    

				while (*point_value != 00)						// send bytes until 00 occure								
					{

						time_outer = time_100ms;
						while (((CTS) || (DSR)) && (time_outer));		// wait to release the line, sometimes it could take up to 1ms
														// to begin transmission it should be = 0	
														// DSR should be set to 0 
														// if something wrong - will not wait longer then 65000 cyccles
														

				 		flag_TxD_finish = 0;					// IF TI->1 means the bit is transmitted
 		
						SBUF=*point_value;						// send byte
			
			       	while(!flag_TxD_finish);			// wait until buffer becomes empty
	
						point_value++;								// pointer to the next byte in message
					}

			TI=0;												// clears interrupt flag
	
//------------ END transfere DATA TO RS232
        
 	      timeout_counter = 30;							// wait some time for OK - 3s
 	      while (timeout_counter);							


			if (!flag_OK) 									// it's normal not to receive response
            {
           	 POWER_pattern 	= LED0;									// dark LEDs									
           	 RING_pattern	= LED0;
             	
             	
				 break;
				}
          }		// end of the loop for 5 times try to power down the system
          
        i = 1;
        POWER_pattern 	= LED4;									// dark LEDs									
        RING_pattern		= LED5;
        
		  if (flag_OK)
	       {

		  		for (;;); 										// lock
		
			 }		// end of loop 5 times
				
		 }	

   if (i)
    flag_noresponse = 1;						// will go to point_F and switch OFF module

	return i;										// if =1 command failed, but device still responding
                                          // if = 0 everything is OK
}




/*************************************************************************/
byte waiter( word mas1,word mas2,byte mas3)	// waits for one of 3 masks to occure
/*************************************************************************/
{

// mask1 and mask 2 are compared with flags
// mask 3 controls <0 0 0 > < RING CHECK> <ring_not_active> <both_pressed_2s> <CLOSE_SW_pressed> <OPEN_SW_pressed>
// 000 in mask 3 is the delay how many x0.2s to wait

/* RESULTS
	0 - mask1 matched
	1 - mask2 matched
	2 - both keys pressed for longer then 2s
	3 - open SW_Pressed
	4 - CLOSE sw pressed
	5 - ring is not anymore active
	6 - DELAY FINISHED, no other event occured
*/
	

	static byte result,mask3,timo;
	static word mask1,mask2;		// redefine parameters to avoid their change

	mask1 = mas1;
	mask2 = mas2;
	mask3 = mas3;
	result = 0xFF;											// this is void result

	OPEN_SW_pressed = 0;									// resets flags for key_pressed
	CLOSE_SW_pressed = 0;								// WAIT FOR NEW KEY PRESSING


	timo = (mask3 >> 4) & 0x0E ;				// take value from the mask3 to wait counter
	timer_waiter = timo;							// if the timer becomes =0 and was not =0 in beginning - break with exit code 6

	check_ring = period_of_check_ringing;								// check ring every 0.5s
	
	while (result == 0xFF && !flag_noresponse && !flag_NO_CARD )  // if something without response - release immediately wait
		{


  
//---------------- check for RING state ------

			if (mask3 & 0x10)				// if flag set to check the RING status
				{
					if (!check_ring)
						{
							check_ring = period_of_check_ringing;	 	// take new value, period of 500ms appeared
							
							
//------------ transfere DATA TO RS232

	point_value = &ATCHECK_STATUS;									// send ATSMSO command    

	while (*point_value != 00)						// send bytes until 00 occure								
		{

			time_outer = time_100ms;
			while (((CTS) || (DSR)) && (time_outer));		// wait to release the line, sometimes it could take up to 1ms
														// to begin transmission it should be = 0	
														// DSR should be set to 0 
														// if something wrong - will not wait longer then 65000 cyccles
														

	 		flag_TxD_finish = 0;					// IF TI->1 means the bit is transmitted
	 		
			SBUF=*point_value;						// send byte
			
       	while(!flag_TxD_finish);			// wait until buffer becomes empty
	
			point_value++;								// pointer to the next byte in message
		}

	TI=0;												// clears interrupt flag
	
//------------ END transfere DATA TO RS232
							

						}
				}


//---------------- check for timeout ---------
			if (timo)						// if timer value was not =0 then wait this time x0.1s and if no other event occured - return code 6
				{
					if (!timer_waiter)
						result=6;
				}

//---------------- mask 2 - lower priority
			if (flags & mask2)			// check for event 2
					result=1;

//---------------- mask 1 - higher priority
			if (flags & mask1)			// check for event 1
					result=0;

          
			if ((mask3 & 0x2) && OPEN_SW_pressed)			// check for both keys pressed longer then 2s
					result=3;

			if ((mask3 & 0x1) && CLOSE_SW_pressed)			// check for both keys pressed longer then 2s
					result=4;
				
			if ((mask3 & 0x8) && (!RING_active))			// check if RING not active
					result = 5;

//--------------- switch ON/OFF operations - highest priority
			if ((mask3 & 0x4) && flag_both_pressed_2s)			// check for both keys pressed longer then 2s
				result=2;
 

			if (check_every_minute)					// every minute the part is auto checked
				{
					command(10,&ATCBC,0);				// send command to check battery if no OK - block
					
					check_every_minute = 0;			// resets the flag for every minute check				
               flag_OK = 0;						// reset OK flag 
				}

		}
	return result;
	
}


/*************************************************************************/
void wait(byte time_wait)  // wait <time_wait> x 100ms time
/*************************************************************************/
{
	timeout_counter = time_wait;
	while(timeout_counter);
	
}



/*************************************************************************/
void TIMER() interrupt 1  // interrupt on timer - manages INDIC routine
/*************************************************************************/
{
	byte i;
	code word *pointek;
	// buttons scan function
	//--------- here passes every 10 ms 
	
	TF0 = 0;					// clears the overflow of the TIMER0
	
	TH0 = 220;					// initial value to produce INT every 10 mS
	TL0 = 0;						// low byte of the value
	

	if (flag_command_received)     
	 {
		
	  COMMANDO = *pointer_TAIL;				// BUFFER DATA, STACK fifo 2 LEVELS
	  flag_command_received--;			// clear flag and calculate 

     if(++pointer_TAIL > &COMMAND_BUFFER[length_of_FIFO-1])
	   pointer_TAIL = &COMMAND_BUFFER[0];

	
	  pointek = &PATTERNS[0];
	  
	  for (i = 0; i< number_of_commands; i++)
	   {
	   	if (*pointek == COMMANDO)
	   	  {
	   	   pointek++;
	   	   flags |= *pointek;		// sets new flags
	   	   break;
	   	  }
	   	
	      pointek++;						// goto nextr address
	      pointek++;
	   
	   }
	   
	   if (HOLD_exception)
	    {
	     if ( *pointek == 0)					// this is the case when _CPAS:3 detected - will set flags only whyle ATHOLD
		 	  flags |= 0x0004;					// no carrier = 1	
		 
		 
		 }

	  
	  	if (i == ring_voice_number)
	  	 {
	  	  ring_timer = ring_time;
	  	  RING_active = 1;
	  	 }
	  		 

	   timer_check_every_minute = one_minute;  // it's about 600 set timer to wait one minute more


/* -------- description of flags
sbit flag_OK = 				flags ^ 8;	// if =1 OK message appeared
sbit flag_ERROR = 			flags ^ 9;	// if =1 ERROR message appeared
sbit flag_NO_CARRIER = 		flags ^10;	// if =1 NO CARRIER message appeared
sbit flag_RING_CHECK = 		flags ^11;	// if =1 the ring is not active anymore
sbit flag_RING_VOICE = 		flags ^12;	// if =1 voice ring 
sbit flag_RING_OTHER = 		flags ^13; 	// if =1 other rings came
sbit flag_NOT_REGISTERED= 	flags ^14;	// if =1 registration in progress, but still not registered
sbit flag_REGISTERED = 		flags ^15;	// if =1 registered in the net
sbit flag_NO_CARD = 			flags ^ 0;	// if =1 no SIM card detection
sbit flag_CARD_INSERTED = 	flags ^ 1;	// if =1 SIM card detected
sbit flag_NO_PIN = 			flags ^ 2 ;	// if =1 no need to enter PIN code
sbit flag_PIN_NEEDED = 		flags ^ 3 ;	// if =1 PIN code needed, should be only 0000
sbit flag_NO_ATTEMPTS =		flags ^ 4 ;	// if =1 no available attempts (1) - don't try anymore
sbit flag_CAN_ATTEMPT = 	flags ^ 5 ;	// if =1 2 or 3 available attempts
*/

	} // end of IF new command appeared


	close = CLOSE_SW;			// read buttons
	open = OPEN_SW;
	
// ------------ detection of key press ---------------------
// these flags are only set, software have to reset them after use
	
	if (!close & old_close)		// close  SW pressed
		CLOSE_SW_pressed = 1;	// set flag
		
	if (!open & old_open)		// open SW pressed
		OPEN_SW_pressed = 1;		// set flag
		
	old_close = close;			// copy new to old values
	old_open = open;

	if (!open && !close)			// check that both keys pressed for longer then 2 s
		{
			if (timer_pressed_2s > 100)   // check if both are hold more then 2s
				{
				  if (!flag_pressed)					// this flag eliminates multiple setting of the variable
					{	
						flag_both_pressed_2s = 1;	// both keys were pressed for longer then 2s					
				   	flag_pressed = 1;				// this flag will disable next setting the flag
				   }
				 }
			  else
  				timer_pressed_2s++;

		}
	  else
	  	{
	  		flag_both_pressed_2s = 0;	// clear flag that both keys pressed for longer then 2s
	  		timer_pressed_2s = 0;		// release timer
	   	flag_pressed = 0;				// this flag will disable next setting the flag

	  	}

	if (++LED_prescaller > 9)					// pre counters for different type of LED control
		{
			// ---------  here passes every 100 ms  -----------------
			// routine that controls LEDs how to light and blink

   		if (time_outer)   						// counter for timeouts
   			time_outer--;	
	
			if (check_ring)
				check_ring--;							// common use counter for detect period of ring check
			
			if (module_ON)
			 {
				if (timer_check_every_minute)
					timer_check_every_minute--;
				  else	
					{
					  timer_check_every_minute = one_minute;  // it's about 600
					  check_every_minute = 1;
					}
			  }
			 else
			  check_every_minute = 0;			 

	      if (flag_RING_CHECK)    			// if receive response that ring is not active - stop immediately the process
	      	ring_timer = 0;		

			if (ring_timer)
				{
					ring_timer--;
					RING_active = 1;			// stil active
				}
			 else
				RING_active = 0;				// clears the flag of ring active			 		

			if (timeout_counter) 			// if timer overflow - stop
				timeout_counter-- ;			// counter used in timeouts, it's 8 bit counter


			if (timer_waiter) 			// if timer overflow - stop
				timer_waiter-- ;			// counter used in timeouts, it's 8 bit counter


//------------------ LED indication procedure - controls lights of LED
// in LED pattern registers is the value of light patterns, that are changed every 100ms
// the whole cycle of all patters  is 1.6s
//----------------------------------------------------------------------

			if (LED_mask & POWER_pattern)
				PORT3_image &= (~0x04);			// clear bit 2 of PORT3_image - LED will light
			  else
			  	PORT3_image |= 0x04;			// sets bit 2 of PORT3_image - LED will dark

			if (LED_mask & RING_pattern)
				PORT3_image &= (~0x20);			// clear bit 5 of PORT3_image - LED will light
			  else
			  	PORT3_image |= 0x20;			// sets bit 5 of PORT3_image - LED will dark
			
			P3 = PORT3_image;					// refreshes outputs

			LED_prescaller = 0;
			LED_mask = LED_mask << 1;		// shift once the mask

			if (!LED_mask)                // mask is no longer valid
				LED_mask = 1;					// load again the mask

		}
			
}

		//end of interrupt routine



/*************************************************************************/
void Serial() interrupt 4		  // interrupt on serial communication
/*************************************************************************/
{
 static byte danni;

//------------ RxD interrupt event occured --------------------	 
 if(RI)				// check if there was comming a character into MPU from serial
						// occures if byte is waiting is SREG to be readed
 	{
	   danni = SBUF & (~0x80);	// remove over 0x80 all characters		

	   if (danni > 0x5F && danni < 0x7B)	// caps lock all characters
			danni &= (~0x20);
										// read data from buffer
	   RI=0;									// clears the flag for comming character to prefent new INT
		RTS = 1;								// diable new comming characters
		P1 = PORT1_image;					// copy data to port
	
//--------------- SETS RTS =0 to disable new comming characters, if the process of transmission
// already began - then only one byte will come in the buffer and uP can read it,
// no way to appear overflow of the register


//============= IMPLEMENTATION of new DALLAS algorythm for decoding commands
	CRC_high = TABLE_HIGH[CRC_high ^ danni];				// recalculate CRC with comming data	
	CRC_low  = TABLE_LOW[CRC_low ^ danni] ^ CRC_high;	// low byte of CRC




//----------- detection of end of comparision - CR LF after 4 incoming character
       if ((danni_old == 0x0D) && (danni == 0x0A))		// check for end of command	 CR LF
	   	{
            if(CRC_high != 0x54 && CRC_low != 0xD5)	// IF cr lf ONLY -SKIP
				  {
				  	*pointer_HEAD = CRC_high*256 + CRC_low;		// after data appeared - copy in protected area
	            flag_command_received++;		// set flag that new command appeared - should be analized
	            if(++pointer_HEAD > &COMMAND_BUFFER[length_of_FIFO-1])
	            	pointer_HEAD = &COMMAND_BUFFER[0];
	            
					flag_debug = 1; //@@@@@@@@@@@@@@@@@@@
				  }

				CRC_high = 0;				// resets the etalons
				CRC_low  = 0;

	   	} // WAS case for delimiter CR LF and results transfered into flags 
	   	

		danni_old = danni;						// new to old value

						// if (RI) became again 1 - new data was comming - don't release RTS and keep it in 0
		
      RTS = 0;	// releases the line and to show to external device that new data could be send again
	 	P1 = PORT1_image;				//sets from image cells to PORT regs
		
						
	 }

//------------ TxD interrupt event occured --------------------	 
 if(TI)                    // occures if the SREG becomes empty, after transmission
 	{
							// if (RI) became again 1 - new data was comming - don't release RTS and keep it in 0
 		TI=0;						// clear Tx interrupt flag to prefent new INT
 		flag_TxD_finish = 1;	// flag that shows that the transfere was finished
 	}

}		//end of interrupt routine




/*************************************************************************/
// 	M A I N    P R O G R A M  - starts after RESET
/*************************************************************************/
void main(void)
/*************************************************************************/
{

static byte i;				// common counter

//-----------DEFINITION  OF THE BRANCH TABLE ------------------------------

#define	POINT_3		0			
#define	POINT_C		1			
#define  POINT_FN		2  
#define  POINT_F		3  
#define	POINT_B		4			
#define	POINT_R		5			
#define  POINT_N		6   // pin support
#define  POINT_N1		7   // pin support
#define  POINT_N2		8   // pin support
#define  POINT_N3		9   // pin support
#define  POINT_NA 	10	 // used in PIN support
#define  POINT_NA1 	11	 // used in PIN support
#define  POINT_NA2 	12	 // used in PIN support
#define	MAIN_LOOP 	13
#define	MAIN_LOOP1 	14
#define	POINT_P		15		
#define	POINT_X		16 	
#define	POINT_X1		17		
#define	POINT_2		18		
#define	POINT_U		19		
#define	POINT_U1		20		
#define	POINT_J		21		
#define	POINT_K		22		
#define	POINT_K1		23		
#define  POINT_L  	24
#define  POWER_OFF	25
	
   pointer_TAIL = &COMMAND_BUFFER[0];
   pointer_HEAD = &COMMAND_BUFFER[0];
	flag_command_received = 0;
	PORT1_image = out0_1;			// all outputs of PORT1 in 0
	PORT3_image = out0_3;			// all outputs of PORT1 in 0, except MUTE and TxD RxD
	DTR = 0;											// set that device is ready
	RTS = 0;		// enable receiving data
	MUTE = 1;							// MUTE = 1
 	P1 = PORT1_image;
	P3 = PORT3_image;		// copy values from image cells to PORT regs
	
//------------ init ports before any other things

	CRC_high = 0;				// resets the etalons
	CRC_low  = 0;
	module_ON = 0;									// module is OFF	
   flag_noresponse = 0;
	module_number = POINT_3;					// in switch module - goto POINT3
	timer_check_every_minute = one_minute;  // it's about 600
 	com_init();							// initialize COM port - 9600 - 8 - 1 - N

	HOLD_exception = 0;		
			
			
for (;;) {           //endless cycle POINT3

	//========= HERE IS THE PLACE TO MAKE GLOBAL DESICIONS
	// TO WATCH IF CARD INSERTED, IF ONLINE, AND SO ON
	
	
	if (flag_NO_CARD && module_number > 5)		// to avoid problems with wrong comming of command
	 {
	 	flags = 0;			// reset flags
	 	// -------- set all outputs in right statuses ---- have to be complete
	   module_number = POINT_N;		// if no card in any moment - go to point N
	 }

	if(flag_noresponse)
	 {
    	POWER_pattern 	= LED4;									// dark LEDs									
     	RING_pattern	= LED5;
	 
	   flag_noresponse = 0;
		module_number = POINT_F;			// was no response from modules
	 }
	 
	                                           // block POWER OFF when active conversation modules 19-23
	if (flag_both_pressed_2s && (module_number>3 && (module_number<19 || module_number>23)))	
		{                                    
		 flag_both_pressed_2s = 0;
		 module_number = POWER_OFF;					// in switch module - goto POINT_C
	   }




//debug(module_number,flags);	//@@@@@@@@@@@@@@@@@@@@@@


 switch (module_number)
	{	

/*************************************************************************/
    	case POINT_3: {
/*************************************************************************/
// ---------- POINT ' 3 " ------------------

    	POWER_pattern 	= LED1;									// dark LEDs									
     	RING_pattern	= LED1;

	flag_ONLINE=0;						// no connection with the NET
	PORT1_image = out0_1;			// all outputs of PORT1 in 0
	PORT3_image = out0_3;			// all outputs of PORT1 in 0, except MUTE and TxD RxD

	ring_timer = 0;					// resets ring_timer;

	DTR = 0;											// set that device is ready
	RTS = 0;		// enable receiving data
	MUTE = 1;							// MUTE = 1
 	P1 = PORT1_image;
	P3 = PORT3_image;							// copy values from image cells to PORT regs

	flag_TxD_finish = 0;				// IF TI->1 means the bit is transmitted
	flag_both_pressed_2s = 0;			// both keys were pressed for longer then 2s					
	flag_ONLINE=1;												// to register in the NET, it could be very long time	

//debug routines @@@@@@@@@@@@@@@@@@@@@
/*
	indic(4,5);		// to ensure that the processor is working

	for (;;)
	 {
	 	while (!flag_debug);
		flag_debug = 0;
	 	debug(0,CODE_COMMAND);
	 }
	 
*/




	module_number = POINT_C;					// in switch module - goto POINT_C

   break;
			} // end of case POINT 3
	  

/*************************************************************************/
		case POINT_C:	{ 
/*************************************************************************/
// ---------- POINT C ----------------

	wait(10);								// wait time 10x100ms

	module_number = POINT_FN;					// in switch module - goto POINT_C
   break;


			} // end of case POINT C
	


/*************************************************************************/
		case POINT_FN:	{ 
/*************************************************************************/
             	POWER_pattern 	= LED0;									// dark LEDs									
             	RING_pattern	= LED0;

	module_number = POINT_F;					// in switch module - goto POINT_C
   break;

}



/*************************************************************************/
		case POINT_F:	{ 
/*************************************************************************/
// ---------- POINT F ----------------

	module_ON = 0;
	flag_noresponse = 0;			// clear flag for single entry from here

   MUTE = 1;         			 // MUTE = 1		
   P3 = PORT3_image;	
	flags = 0;
	while (!flag_both_pressed_2s);	// wait while both switches are pressed togather for more then 1 s 

	module_ON = 1;							// SWITCH on THE MODULE FLAG
	flag_both_pressed_2s = 0;

	flags = 0;							// reset all RS232 received messages flags

	module_number = POINT_B;					// in switch module - goto POINT_C
   break;
			} // end of case POINT F
			

/*************************************************************************/
		case POINT_B:	{ 
/*************************************************************************/
// ---------- POINT B ----------------
// flags=0 are not included in this cycle, to allow program to pass with flag_OK=1
// after returning from other pranches

	IGNITION = 1;					
	POWER_ON = 1;
	P1 = PORT1_image;
	P3 = PORT3_image;
						// copy values from image cells to PORT regs
						
  	POWER_pattern 	= LED2;										
  	RING_pattern	= LED0;			// xxxx zzzz  x - patterns for POWER  z-patterns for RING	


	if (command(10,&EMPTY,0))				//  wait for OK 3s, sending nothing. just waiting for OK
        flag_noresponse = 1;				// will go to point_F and switch OFF module
	  else
		 {
		   if (!GSM_ON)
	        flag_noresponse = 1;				// will go to point_F and switch OFF module
		  }	


  	module_number = POINT_R;				// in switch module - goto POINT_R
   break;
			} // end of case  POINT B
			
			
/*************************************************************************/
		case POINT_R:	{ 
/*************************************************************************/
// ---------- POINT R ----------------
	flags = 0;							// resets all flags again
	IGNITION = 0;					
	MUTE = 0;
	P1 = PORT1_image;
	P3 = PORT3_image;
					// copy values from image cells to PORT regs
					
  	POWER_pattern 	= LED2;										
  	RING_pattern	= LED0;			// xxxx zzzz  x - patterns for POWER  z-patterns for RING	
	module_number = POINT_N;	// in switch module - goto POINT_N
	break;	
	
	}
		

/*************************************************************************/
		case POINT_N:	{ 
/*************************************************************************/
// --------- POINT "N" // new support for PIN code
// - waits for card inserted
	
  	POWER_pattern 	= LED2;
  	RING_pattern	= LED0;
									   // blinks led	


   flags = 0;
 	waiter(mask_EXISTS_SIM,mask_EXISTS_SIM,mask_POWER_OFF); 		// wait forever for POWER off or SIM exists

	module_number = POINT_N1;		// in switch module - goto POINT_3
	break;								// go to POINT_3, skip POINT_B loop
 			
 			
	} // end of case POINT_N



/*************************************************************************/
		case POINT_N1:	{ 
/*************************************************************************/
// to bring some way to check if power off pressed

	//------- sim card inserted ?

	flags = 0;			// clear all flags	

	command(middle_cmnd_time,&ASK_FOR_PIN,0);					// send ASK fro existing of PIN

	module_number = POINT_N2;			// continue next point
	break;
	
	}		// end of the case POINT_N1
	


/*************************************************************************/
		case POINT_N2:	{ 
/*************************************************************************/


 	waiter(mask_NO_PIN_CODE+mask_PIN_CODE_NEED,0,mask_POWER_OFF); 		// wait forever for POWER off or SIM exists


	if (flag_NO_PIN || flag_both_pressed_2s)	// no pin code needed or power OFF
		{				
         flags &= clear_flags;						// clear flags
	   	module_number = POINT_NA; 		// goto POINT_NA where continue normal process
	   	break;
		}
		
// now - PIN support module

		flags = 0;
		
		command(middle_cmnd_time,&ASK_PIN_ATTEMPTS,0);					// send ASK how many attempts last
		module_number = POINT_N3;			// goto next number
		break;		// if wrong operation - goto pointF
		

	} // end of case N2


/*************************************************************************/
		case POINT_N3:	{ 
/*************************************************************************/
// eliminates need to check for both pressed

																			// wait forever for POWER off or SIM exists

	if (waiter(mask_NO_ATTEMPTS, mask_CAN_ATTEMPT,mask_POWER_OFF) == 0)			// no attempts last
	 {
		waiter(mask_NO_SIM,0,mask_POWER_OFF); 		// wait forever for POWER off or SIM exists
		module_number = POINT_N;		// in switch module - goto POINT_3
		break;								// go to POINT_3, skip POINT_B loop

		// if power off appeared - will be executed automatiocaly on the entry of the master loop
	
    }


		flags = 0;	

		if (!command(long_cmnd_time,&ENTER_PIN_CODE,3))	 // send enter PIN code, 
									// addition in command - option 3 means - send it only once and wait 3s
		{ 
		  if (!flag_ERROR)	
        	{
        	 flags = 0;
   	 	 module_number = POINT_NA; 	/// goto POINT_NA if good response
		 	 break;
         }
		}

		 if (flag_ERROR)				// if returns error - code is wrong
		    flag_noresponse = 0;		// clear error flag to avoid POWER OFF module
 
	
		flags = 0;			
		waiter(mask_NO_SIM,0,mask_POWER_OFF); 		// wait forever for POWER off or SIM exists
												// if power off -will go automatically
		module_number = POINT_N;		// in switch module - goto POINT_3
		break;								// go to POINT_3, skip POINT_B loop
	

	}	// end of the case N		



/*************************************************************************/
		case POINT_NA:	{ 
/*************************************************************************/
// ---------- POINT NA ----------------

	waiter(mask_REGISTERED,0,mask_POWER_OFF);				// wait until <CREG: 1> message appears or both pressed - POWER OFF
																	// set flag that is ONLINE, if disconnection detected - will be reset

	//-------- flag online could be set only here, and reset on any other place or time,
	//         resulting that the program will flow back to PONT_B				


		module_number = POINT_NA1;		// in switch module -
		break;								// 

  } // END OF THE CASE NA

/*************************************************************************/
		case POINT_NA1:	{ 
/*************************************************************************/
	// ========= SETTING THE AUDIO FEATURES ===========
	
	command(20,&ATVIP,0); 					// SET IN EXTENDED AUDIO MODE
	flag_OK = 0;										// resets flags for OK

	wait(20);										// wait 3s

	module_number = POINT_NA2;		// in switch module - goto POINT_3
	break;								// go to POINT_3, skip POINT_B loop

 }
 
/*************************************************************************/
		case POINT_NA2:	{ 
/*************************************************************************/
 
	command(20,&ATSETUP,0);                // set parameters
	flag_OK = 0;									// resets all flags

	command(20,&ATCBC,0);                  // set parameters
	flags = 0;									   // resets all flags

	flag_ONLINE=1;												// to register in the NET, it could be very long time	

	
	module_number = MAIN_LOOP;					// in switch module - goto POINT_C
   break;
			} // end of case POINT R


			
/*************************************************************************/
		case MAIN_LOOP:	{ 
/*************************************************************************/
// ----------  POINT - MAIN_LOOP	-----------------------------------
// ------------- check for ASYNC/SYNC CRING operations

	flags &= clear_flags;					// resets all flags
	
  	POWER_pattern 	= LED1;										
  	RING_pattern	= LED0;
	
	
	
					// light POWER LED, dark RING


	waiter(mask_ring_no_voice+mask_ring_voice,0,mask_POWER_OFF);	// check if RING  occurred  or POWER 0FF

	module_number = MAIN_LOOP1;		
	break;
}	
 
 
/*************************************************************************/
		case MAIN_LOOP1:	{   	// continue if main loop
/*************************************************************************/
 
	if (!flag_ONLINE)							// check if disconnection occured
		{
			module_number = POINT_R;		// in switch module - goto POINT_3
		   break;
		}
		   			
//----- if not CRING VOICE occured (other RINGS)					

  	if (flag_RING_OTHER)									// check if ring_no voice received
  		{
			module_number = POINT_L;		// in switch module - goto POINT_L
 			break;								// go to POINT_L
  		}
  		
			// ------------- VOICE call ----------------

	flags = 0;												// resets all flags

  	RING_pattern	= LED2;								// RING led is blinking, power no change

	module_number = POINT_P;					// in switch module - goto POINT_C
   break;
			} // end of  MAIN_LOOP



/*************************************************************************/
		case POINT_L:	{ 
/*************************************************************************/
// ----------- POINT L --------------------------
  		
	  	if(command(cmnd_time,&ATH,1))    // unrecoverale error occured goto POINT B
				break;							// if error
		  else
		  	 {
		  	 	if (!flag_ONLINE)
		  		  {	
		  			flags = 0;
		  			module_number = POINT_R;
		  			break;
		  		  }	
			  		  
			  	RING_pattern	= LED0;							// RING_LED dark
				flags=0;									// clear all flags	

				module_number = MAIN_LOOP;	// goto main loop again
				break;
           }
    } // end of case  POINT L




/*************************************************************************/
		case POINT_P:	{ 
/*************************************************************************/
// ------------- POINT P  ----------------

	i = waiter(mask_NO_CONNECTION+mask_NO_CARRIER,0,0b00011011); 		// wait until one of the keys pressed and if OPEN do the following
													// !!!!! here checks every 0.5s for ring status xxx1xxxx - in mask3


	if (!flag_ONLINE)			// check if disconnection occured
		 {
		   module_number = POINT_R;
		   break;
		 }


	if (flag_NO_CARRIER || (!RING_active))				// check if the call terminated or no ringing anymore
		{
			flags = 0;											// clear all flags
			
		  	RING_pattern	= LED0;			// RING LED DARK
		   module_number = MAIN_LOOP; 					/// goto MAIN LOOP
		   break;
		}

	if ( i == 3)
      {
		   module_number =POINT_X; 					/// goto POINT_X i = 3
		   break;
		 }
		else
		 {
		   module_number =POINT_L ; 					/// goto POINT_X i != 3
		   break;
		  }
	 	
	} // end of case  POINT _ P



/*************************************************************************/
		case POINT_X:	{ 
/*************************************************************************/
//---------------- POINT " X " ------------------
		
      	wait(1);
			command(cmnd_time,&ATA,2);					// send command ATA
			// --- command ATA successfully transmited

			if (!flag_ONLINE)							// check if disconnection occured
				{	
				   module_number =POINT_R; 					/// goto POINT_R
				   break;
				}         	 
			   
			if (flag_ERROR)							// if error flag occured - ring was stopped
				{
					module_number = MAIN_LOOP;
				  	RING_pattern	= LED0;		// dark RING LED
					break;                        // goto main loop point
				}
		  	RING_pattern	= LED1;				// RING LED ON

		   module_number =POINT_X1; 					
		   break;
 }


/*************************************************************************/
		case POINT_X1:	{ 
/*************************************************************************/

// -------------- DELAY MODULE TO ENSURE THAT KEY PRESS ARE NOT VERY OFTEN

		 	waiter(mask_NO_CONNECTION,mask_NO_CARRIER,0b01100000); 		// wait until 0.5s or no connection detected

			if (flags & mask_NO_CARRIER)
				{
				   module_number = MAIN_LOOP;      //  goto MAIN_LOOP
				   break;
				}
					
			if (flags & mask_NO_CONNECTION)
				{
				   module_number = POINT_R;      //  goto POINT_R - NO CONNECTION
				   break;
				}


		   module_number =POINT_2; 					/// goto POINT_2
		   break;
         
	} // end of case POINT_X1



/*************************************************************************/
		case POINT_2:	{ 
/*************************************************************************/
//---------------- POINT " 2 " ------------------

			flags &= mask_NO_CARRIER;					// resets all flags, except NO_carrier
			waiter(mask_NO_CARRIER,mask_NO_CONNECTION,0b00000011);				// wait for key pressed or no_carrier   		
			

			if (!flag_ONLINE)							// check if disconnection occured
				{	
				   module_number =POINT_R; 					/// goto POINT_R
				   break;
				}         	 

			if (flag_NO_CARRIER)							// check if the call terminated
				{
					flags = 0;											// clear all flags
				  	RING_pattern	= LED0;

				   module_number = MAIN_LOOP; 					/// goto main loop
				   break;
				}

			if (OPEN_SW_pressed)   		
				{
				   module_number = POINT_U; 					/// open /hold line
				   break;
				}
			  else
				{
				   module_number = POINT_L; 					/// close line
				   break;
				}
			  		
	} // end of case  POINT_2


/*************************************************************************/
		case POINT_U:	{ 
/*************************************************************************/
//------------- POINT "U" ------------------
					command(cmnd_time,&ATHOLD,1);								// send AT+CHLD=2 comman
      			// command CMND_HOLD transmitedd successfully	             	

         
//---------------- POINT " T " ------------------
					if (!flag_ONLINE)							// check if disconnection occured
		          {
					   module_number =POINT_R; 					/// goto POINT_R
					   break;
					 }
					 
			  		RING_pattern	= LED3;		// RING LED blinks 2x0.1 and 0.5 pause
					flags &= mask_NO_CARRIER;				// resets all flags, except NO_carrier

				   module_number = POINT_U1; 					/// close line
				   break;

  }		// end of case U


/*************************************************************************/
		case POINT_U1:	{ 
/*************************************************************************/
// -------------- DELAY MODULE TO ENSURE THAT KEY PRESS ARE NOT VERY OFTEN

				 	waiter(mask_NO_CONNECTION,mask_NO_CARRIER,0b01100000); 		// wait until 0.5s or no connection detected

					if (flags & mask_NO_CARRIER)
						{
						   module_number = MAIN_LOOP;      //  goto MAIN_LOOP
						   break;
						}
					
					if (flags & mask_NO_CONNECTION)
						{
						   module_number = POINT_R;      //  goto POINT_R - NO CONNECTION
						   break;
						}

				   module_number =POINT_J; 					/// goto POINT_R
				   break;

		
	} // end of case POINT_U


/*************************************************************************/
		case POINT_J:	{ 
/*************************************************************************/
// --------- POINT "J"

					waiter(mask_NO_CARRIER,mask_NO_CONNECTION,0b00000011);				// wait for key pressed or no_carrier   		

					if (!flag_ONLINE)							// check if disconnection occured
		          {
					   module_number =POINT_R; 					/// goto POINT_R
					   break;
					 }

					if (flag_NO_CARRIER)
						{
							flags = 0;											// clear all flags
							
  							RING_pattern	= LED0;				// RING LED DARK
						   module_number = MAIN_LOOP; 					/// goto main loop
						   break;
						}


               if (OPEN_SW_pressed)
               	{

						   module_number = POINT_K; 					/// goto POINT_K
						   break;
                  }
                else
                  {
						   module_number = POINT_L; 					/// goto POINT_L
						   break;
                   }

	} // end of case  point J
	
/*************************************************************************/
		case POINT_K:	{ 
/*************************************************************************/
// --------- POINT "K"

              command(cmnd_time,&ATHOLD,1);
      			// command ATHOLD transmited OK

					if (!flag_ONLINE)							// check if disconnection occured
					 { 
					   module_number = POINT_R; 					/// goto POINT_R
					   break;
					  }
					RING_pattern	= LED1;				// LIGHT RING LED

				   module_number = POINT_K1; 					/// goto POINT_K1
				   break;

 }
 
 
/*************************************************************************/
		case POINT_K1:	{ 
/*************************************************************************/
// -------------- DELAY MODULE TO ENSURE THAT KEY PRESS ARE NOT VERY OFTEN

		 	waiter(mask_NO_CONNECTION,mask_NO_CARRIER,0b01100000); 		// wait until 0.5s or no connection detected

			if (flags & mask_NO_CARRIER)
				{
				   module_number = MAIN_LOOP;      //  goto MAIN_LOOP
				   break;
				}
					
			if (flags & mask_NO_CONNECTION)
				{
				   module_number = POINT_R;      //  goto POINT_R - NO CONNECTION
				   break;
				}

      			      
						   module_number = POINT_2; 					/// goto POINT_2
						   break;
      			      
	} // end of case  point K
	

/*************************************************************************/
		case POWER_OFF:	{ 
/*************************************************************************/
// --------- POINT "POWER_OFF"

  	POWER_pattern 	= LED2;										
  	RING_pattern	= LED0;								// 13.11.2000 corection - when power down - blink POWER LED

	IGNITION = 0;
	MUTE = 1;												// to avoid noice in Audio track

	P3 = PORT3_image;
	P1 = PORT1_image;

	wait(5);													// wait 0.5 s to ensure that everything is OK
	
	POWER_ON = 0;				
	P1 = PORT1_image;

 	
	 for (i=0;i<5;i++)
	 	{

//------------ transfere DATA TO RS232

	point_value = &ATSMSO;									// send ATSMSO command    

	while (*point_value != 00)						// send bytes until 00 occure								
		{

			time_outer = time_100ms;
			while (((CTS) || (DSR)) && (time_outer));		// wait to release the line, sometimes it could take up to 1ms
														// to begin transmission it should be = 0	
														// DSR should be set to 0 
														// if something wrong - will not wait longer then 65000 cyccles
														

	 		flag_TxD_finish = 0;					// IF TI->1 means the bit is transmitted
	 		
			SBUF=*point_value;						// send byte
			
       	while(!flag_TxD_finish);			// wait until buffer becomes empty
	
			point_value++;								// pointer to the next byte in message
		}

	TI=0;												// clears interrupt flag
	
//------------ END transfere DATA TO RS232



         timeout_counter = 30;						// wait - 1.5s for GSM_ON = 0 
         while (timeout_counter);							


			flags=0;
/*-------- new algorythm for shut down
  1. send command for shut down
  2. wait 1.5s for GSM_ON to become 0
  3. try this 5 times if not shut down
  
*/

			if (!GSM_ON) 									// check for shut down
				{
				  	POWER_pattern 	= LED0;										
  					RING_pattern	= LED0;
	
					break;                           // break the loop
				}	
		}		
     
     		if (i==5)									// if device not shut down properly 5 times
			  {	
			  	POWER_pattern 	= LED4;										
  				RING_pattern	= LED5;
     		  }
     			
	   	module_number = POINT_F; 					/// goto POINT_F
	   	break;
		


	} // end of case  point Z


		
		

 	
 } // end of switch command

}	// endless loop to maintain branch table
 
} // end of main program
           
